"""
Model base module.

This module provides a base class for all models in the Neuralk AI SDK.
"""



class ModelBase:
    """
    Base class for all models in the Neuralk AI SDK.
    """

    def __init__(self):
        """
        Initialize the base model.
        """
        pass
