"""
Core model objects used in the SDK API.

This module defines the primary data structures and classes that represent key concepts
manipulated by the SDK. These objects are typically used for configuration, serialization,
and interaction with the underlying services.

"""

from ..utils.docs import add_submodules_to_docstring


add_submodules_to_docstring(__name__)
